package com.simuwang.daq.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.simuwang.base.common.conts.DateConst;
import com.simuwang.base.pojo.CoreMailAttachmentDTO;
import com.simuwang.base.pojo.dos.CoreMailCookieSidDTO;
import com.simuwang.base.pojo.dos.CoreMailFolderDTO;
import com.simuwang.base.pojo.dos.CoreMailInfoDTO;
import com.simuwang.base.pojo.dto.EmailContentInfoDTO;
import com.simuwang.base.pojo.dto.MailboxInfoDTO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

@Service
public class ReadEmailService {

    private static final Logger log = LoggerFactory.getLogger(ReadEmailService.class);

    private final CoreMailApiService coreMailApiService;

    public ReadEmailService(CoreMailApiService coreMailApiService) {
        this.coreMailApiService = coreMailApiService;
    }

    /**
     * 通过接口获取邮件
     *
     * @param mailboxInfoDTO 邮箱账号密码
     * @param startDate      开始日期
     * @param endDate        结束日期
     * @return 邮件信息
     */
    public Map<String, List<EmailContentInfoDTO>> readEmail(MailboxInfoDTO mailboxInfoDTO, Date startDate, Date endDate) {
        Map<String, List<EmailContentInfoDTO>> emailKeyFileMap = MapUtil.newHashMap();
        String account = mailboxInfoDTO.getAccount();
        String password = mailboxInfoDTO.getPassword();
        // 获取token
        CoreMailCookieSidDTO cookieSidDTO = coreMailApiService.login(account, password);
        if (cookieSidDTO == null || cookieSidDTO.getSid() == null) {
            log.error("邮箱登录失败, 无法获取到token");
            return emailKeyFileMap;
        }
        String token = cookieSidDTO.getSid();
        String cookie = cookieSidDTO.getCookie();
        // 获取邮箱文件夹(默认只读取收件箱Inbox的邮件)
        List<CoreMailFolderDTO> mailFolderList = coreMailApiService.getAllFolders(token, cookie);
        Integer folderId = CollUtil.isEmpty(mailFolderList) ? 1 : mailFolderList.stream().filter(e -> e.getName().equals("收件箱"))
                .findFirst().map(CoreMailFolderDTO::getId).orElse(1);
        // 获取邮件
        List<CoreMailInfoDTO> coreMailInfoDTOList = coreMailApiService.getFolderMails(token, cookie, folderId);
        // 过滤符合解析要求的邮件
        List<CoreMailInfoDTO> readEmailDTOList = CollUtil.newArrayList();
        for (CoreMailInfoDTO coreMailInfoDTO : coreMailInfoDTOList) {
            String emailDate = DateUtil.format(coreMailInfoDTO.getSentDate(), DateConst.YYYY_MM_DD_HH_MM_SS);
            if (!coreMailInfoDTO.getFid().equals(folderId)) {
                log.info("非收件箱邮件,不解析 -> 邮件主题:{}, 日期:{}", coreMailInfoDTO.getSubject(), emailDate);
                continue;
            }
            if (!filterBySubject(coreMailInfoDTO.getSubject())) {
                log.info("主题不满足解析条件 -> 邮件主题:{}, 日期:{}", coreMailInfoDTO.getSubject(), emailDate);
                continue;
            }
            if (!filterByEmailDate(coreMailInfoDTO.getReceivedDate(), startDate, endDate)) {
                log.info("日期不满足解析条件 -> 邮件主题:{}, 日期:{}", coreMailInfoDTO.getSubject(), emailDate);
                continue;
            }
            readEmailDTOList.add(coreMailInfoDTO);
        }

        if (CollUtil.isEmpty(readEmailDTOList)) {
            log.info("获取不到满足解析的条件邮件");
            return emailKeyFileMap;
        }
        Map<String, CoreMailInfoDTO> emailIdInfoMap = coreMailInfoDTOList.stream().collect(Collectors.toMap(CoreMailInfoDTO::getId, e -> e));
        // 根据邮件id获取邮件内容
        List<String> mailIdList = readEmailDTOList.stream().map(CoreMailInfoDTO::getId).toList();
        for (String mailId : mailIdList) {
            CoreMailInfoDTO coreMailInfoDTO = emailIdInfoMap.get(mailId);
            String emailDate = DateUtil.format(coreMailInfoDTO.getSentDate(), DateConst.YYYY_MM_DD_HH_MM_SS);
            String mid = mailId.split(":")[1];
            // 获取邮件附件id(一封邮件可能有多个附件)
            List<CoreMailAttachmentDTO> attachmentDTOList = coreMailApiService.getAttachmentOfMail(token, cookie, mid);
            if (CollUtil.isEmpty(attachmentDTOList)) {
                log.info("邮件没有附件,不解析 -> 邮件主题:{}, 日期:{}", coreMailInfoDTO.getSubject(), emailDate);
                continue;
            }
            List<EmailContentInfoDTO> contentInfoDTOList = CollUtil.newArrayList();
            // 根据附件id下载附件
            for (CoreMailAttachmentDTO coreMailAttachmentDTO : attachmentDTOList) {
                int attachmentId = coreMailAttachmentDTO.getId();
                String filename = coreMailAttachmentDTO.getFilename();
                if (StrUtil.isBlank(filename)) {
                    log.info("邮件附件名称为空 -> 邮件主题:{}, 日期:{}", coreMailInfoDTO.getSubject(), emailDate);
                    continue;
                }
                File file = coreMailApiService.getFileFromAttachmentMail(token, cookie, mid, attachmentId, filename);
                if (file == null) {
                    log.error("邮件附件下载失败 -> mid:{}, 邮件主题:{}, 日期:{}", mid, coreMailInfoDTO.getSubject(), emailDate);
                    continue;
                }
                EmailContentInfoDTO contentInfoDTO = new EmailContentInfoDTO();
                contentInfoDTO.setEmailAddress(account);
                contentInfoDTO.setEmailTitle(coreMailInfoDTO.getSubject());
                contentInfoDTO.setEmailDate(emailDate);
                contentInfoDTO.setFileName(filename);
                contentInfoDTO.setFilePath(file.getAbsolutePath());
                contentInfoDTO.setSenderEmail(extractEmail(coreMailInfoDTO.getFrom()));
                contentInfoDTO.setEmailType(1);
                contentInfoDTOList.add(contentInfoDTO);
            }
            if (CollUtil.isNotEmpty(contentInfoDTOList)) {
                emailKeyFileMap.put(mid, contentInfoDTOList);
            }
        }
        return emailKeyFileMap;
    }

    private String extractEmail(String address) {
        if (StrUtil.isEmpty(address)) {
            return "";
        }
        Pattern pattern = Pattern.compile("<([^>]+)>");
        Matcher matcher = pattern.matcher(address);
        if (matcher.find()) {
            return matcher.group(1);
        } else {
            return address.trim();
        }
    }

    private boolean filterByEmailDate(Date receivedDate, Date startDate, Date endDate) {
        return receivedDate != null && receivedDate.after(startDate) && receivedDate.before(endDate);
    }

    private boolean filterBySubject(String subject) {
        return StrUtil.isNotBlank(subject)
                && (subject.contains("净值") || subject.contains("业绩报酬") || subject.contains("业绩提成"));
    }
}
